package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.time.LocalDateTime;
import java.util.List;

import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.SysRole;
import com.example.civilizedcity.entity.SysUser;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.SysOrg;
import com.example.civilizedcity.service.SysOrgService;

/**
 * 单位表;(sys_org)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "单位表对象功能接口")
@RestController
@RequestMapping("/")
public class SysOrgController extends BaseController {

    @Autowired
    private SysOrgService sysOrgService;

    /**
     * 通过ID查询单条数据
     *
     * @param orgId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/sysOrg/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(sysOrgService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/sysOrg")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("单位名称") @RequestParam(value = "name", required = false) String name,
                             @ApiParam("是否责任单位") @RequestParam(value = "isResponsible", required = false) Integer isResponsible) throws Exception {

        IPage<SysOrg> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "name", name);
        queryWrapper.eq(null != isResponsible, "is_responsible", isResponsible);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByAsc("sort_no");
        IPage<SysOrg> result = sysOrgService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param sysOrg 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/sysOrg")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody SysOrg sysOrg) throws Exception {
        if (StringUtils.isEmpty(sysOrg.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        if (StringUtils.isEmpty(sysOrg.getOrgPId())) {
            return ResponseBean.error("父级节点不能为空");
        }

        long i = sysOrgService.countBy("name", sysOrg.getName(), false);
        if (i > 0) {
            return ResponseBean.error("名称已存在");
        }

        // 构造 org 编码
        long maxCode = sysOrgService.getMaxCodeOfParent(sysOrg.getOrgPId());
        sysOrg.setOrgCode(String.valueOf(maxCode + 1));

        sysOrg.setOrgId(null);
        sysOrg.setStatus(Constants.STATUS_NORMAL);
        sysOrg.setCreateDate(LocalDateTime.now());
        sysOrgService.save(sysOrg);
        return ResponseBean.success(sysOrg);
    }

    /**
     * 更新数据
     *
     * @param sysOrg 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/sysOrg/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody SysOrg sysOrg,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        if (StringUtils.isEmpty(sysOrg.getName())) {
            return ResponseBean.error("名称不能为空");
        }

        SysOrg exists = sysOrgService.getByButNot("name", sysOrg.getName(), "org_id", id, false);
        if (null != exists) {
            return ResponseBean.error("名称已存在");
        }

        // 比较是否更新了父节点
        SysOrg origin = sysOrgService.getById(id);
        if (!origin.getOrgPId().equals(sysOrg.getOrgPId())) {
            // 如果更换了
            long maxCode = sysOrgService.getMaxCodeOfParent(sysOrg.getOrgPId());
            String newCode = String.valueOf(maxCode + 1);
            // 当前节点以及子节点，全部更换
            sysOrgService.changeOrgCode(origin.getOrgCode(), newCode);
            sysOrg.setOrgCode(newCode);
        }

        sysOrg.setOrgId(id);
        sysOrgService.updateById(sysOrg);
        return ResponseBean.success(sysOrg);
    }

    /**
     * 通过主键删除数据
     *
     * @param orgId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/sysOrg/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        SysUser sysUser = currentUser();

        if (Constants.ROOT_ID.equals(id)) {
            return ResponseBean.error("系统机构, 禁止删除");
        }

        long count = sysOrgService.countBy("org_p_id", id, true);
        if (count > 0) {
            return ResponseBean.error("请先删除子级单位");
        }

        if (!Constants.ROOT_ORG.equals(sysUser.getOrgId())) {
            return ResponseBean.error("暂无权限");
        }

        sysOrgService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}