package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.*;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaCheck;
import com.example.civilizedcity.service.TaCheckItemService;
import com.example.civilizedcity.service.TaCheckService;
import com.example.civilizedcity.vo.CheckLocExport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 模拟测评;(ta_check)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "模拟测评对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckController extends BaseController {

    @Value("${yz.excel.tpl.check}")
    String tplPath;
    @Autowired
    private TaCheckService taCheckService;
    @Autowired
    private TaCheckItemService taCheckItemService;

    /**
     * 通过ID查询单条数据
     *
     * @param checkId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheck/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taCheckService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheck")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("测评名称") @RequestParam(value = "title", required = false) String title,
                             @ApiParam("是否有效") @RequestParam(value = "isValid", required = false) Boolean isValid,
                             @ApiParam("开始日期") @RequestParam(value = "startDate", required = false) String startDate,
                             @ApiParam("结束日期") @RequestParam(value = "endDate", required = false) String endDate) throws Exception {

        String today = DateUtils.today();

        IPage<TaCheck> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaCheck> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(title), "title", title);
        queryWrapper.nested(null != isValid && isValid, q -> q.le("start_date", today).and(f -> f.ge("end_date", today)));

        if (!StringUtils.isEmpty(startDate)) {
            queryWrapper.le("start_date", endDate);
            queryWrapper.ge("end_date", startDate);
        }

        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");
        IPage<TaCheck> result = taCheckService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheck 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheck")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheck taCheck) throws Exception {
        taCheckService.save(taCheck);

        // 创建成功之后, 再创建
        taCheckItemService.createNewItems(taCheck);

        return ResponseBean.success(taCheck);
    }

    /**
     * 更新数据
     *
     * @param taCheck 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheck/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheck taCheck,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taCheck.setCheckId(id);
        taCheckService.updateById(taCheck);
        return ResponseBean.success(taCheck);
    }


    /**
     * 导出实地测评
     *
     * @param taCheck 实例对象
     * @return 实例对象
     */
    @ApiOperation("导出实地测评")
    @PostMapping("/taCheck/{id}/loc/export")
    public ResponseBean exportLoc(@ApiParam("对象ID") @PathVariable String id,
                                  HttpServletResponse response) throws Exception {
        TaCheck taCheck = taCheckService.getById(id);
        List<CheckLocExport> list = taCheckItemService.getExportByCheck(id, Constants.CHECK_OF_LOC);
        if (null == list) {
            return ResponseBean.error("当前测评数据不正确");
        }

        Double totalFullScore = 0.0;
        Integer totalAnswerNum = 0;
        Integer totalNum = 0;
        Double totalSubtotal = 0.0;
        Double totalScore = 0.0;

        for (CheckLocExport item : list) {
            totalFullScore += getVal(item.getFullScore());
            totalAnswerNum += getVal(item.getAnswerNum());
            totalNum += getVal(item.getNum());
            totalSubtotal += getVal(item.getSubtotal());
            totalScore += getVal(item.getScore());
        }

        String title = taCheck.getTitle() + " 实地测评";
        Map<String, Object> map = new HashMap<>();
        map.put("totalFullScore", totalFullScore);
        map.put("totalAnswerNum", totalAnswerNum);
        map.put("totalNum", totalNum);
        map.put("totalSubtotal", totalSubtotal);
        map.put("totalScore", totalScore);
        map.put("title", title);

        InputStream tpl = new FileInputStream(tplPath);
        ExcelUtils.flushFile(response, tpl, CheckLocExport.class, list, map, title);

        return null;
    }

    private Double getVal(Double d) {
        return null == d ? 0.0 : d;
    }

    private Integer getVal(Integer d) {
        return null == d ? 0 : d;
    }


    /**
     * 复制
     *
     * @param id 实例ID
     * @return 实例对象
     */
    @ApiOperation("复制")
    @PutMapping("/taCheck/{id}/copy")
    public ResponseBean copy(@ApiParam("测评ID") @PathVariable String id,
                             @ApiParam("对象实体") @RequestBody TaCheck taCheck) throws Exception {
        TaCheck origin = taCheckService.getById(id);
        if (null == taCheck || origin.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到模拟测评信息");
        }

        String title = StringUtils.ifNull(taCheck.getTitle(), origin.getTitle() + "-复制");
        TaCheck exists = taCheckService.getExistBy("title", title, false, true);

        if (null != exists) {
            return ResponseBean.error("测评名称重复");
        }

        SysUser sysUser = currentUser();
        origin.setCreateUser(sysUser.getUserId());
        origin.setTitle(title);
        TaCheck newCheck = taCheckService.copy(origin);
        return ResponseBean.success(newCheck);
    }


    /**
     * 计算分数
     *
     * @param id 实例ID
     * @return 实例对象
     */
    @ApiOperation("计算分数")
    @PutMapping("/taCheck/{id}/score")
    public ResponseBean calcScore(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaCheck taCheck = taCheckService.getById(id);
        if (null == taCheck || taCheck.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到模拟测评信息");
        }

        taCheck = taCheckService.calcScore(taCheck);
        return ResponseBean.success(taCheck);
    }

    /**
     * 通过主键删除数据
     *
     * @param checkId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheck/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckService.removeLogicById(id);
        return ResponseBean.success("success");
    }

    /**
     * 同步点位分类
     *
     * @checkId
     */

    @ApiOperation("通过id来同步点位")
    @PutMapping("/taCheck/{checkId}/sync")
    public ResponseBean syncPoint(@ApiParam("对象ID") @PathVariable String checkId) {
        taCheckItemService.syncLoc(checkId);
        taCheckItemService.updateLoc(checkId);
        return ResponseBean.success("success");
    }
}